#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define _USE_MATH_DEFINES
#include <math.h>
#include "ido.h"


void   swap
// ====================================================================
//
// purpos     :  pointer swap *f and *g
//
// date       :  Mar 01, 2009
// programmer :  Takayuki Aoki
// place      :  Tokyo Institute of Technology
//
(
   float   **f,         /* array pointer f                           */
   float   **g          /* array pointer g                           */
)
// --------------------------------------------------------------------
{
     float  *tmp = *f;  *f = *g;   *g = tmp;
}



void   malloc_variables
// ====================================================================
//
// purpos     :  dynamic memory allocation (x, y, xn, yn)
//
// date       :  Mar 01, 2009
// programmer :  Takayuki Aoki
// place      :  Tokyo Institute of Technology
//
(
   int      numGPUs,    /* available number of GPU device            */
   int      n,          /* number of particles                       */
   float    **x,        /* x-coordinate of the particles             */
   float    **y,        /* y-coordinate of the particles             */
   float    **xn,       /* updated x-coordinate of the particles     */
   float    **yn        /* updated y-coordinate of the particles     */
)
// --------------------------------------------------------------------
{
     if(numGPUs > 0) {
        cudaMalloc( (void**) x,  n*sizeof(float));
        cudaMalloc( (void**) y,  n*sizeof(float));
        cudaMalloc( (void**) xn, n*sizeof(float));
        cudaMalloc( (void**) yn, n*sizeof(float));
     }
     else {
        *x  = (float *) malloc(sizeof(float)*n);
        *y  = (float *) malloc(sizeof(float)*n);
        *xn = (float *) malloc(sizeof(float)*n);
        *yn = (float *) malloc(sizeof(float)*n);
     }
}



void   r8img_bmp
// ====================================================================
//
// purpos     :  generation of BMP image file (2-Dimensional)
//
// date       :  Mar 01, 2009
// programmer :  Takayuki Aoki
// place      :  Tokyo Institute of Technology
//
(
   int      numGPUs,    /* available number of GPU device            */
   int      n,          /* number of particles                       */
   float    *x,         /* x-coordinate of the particles             */
   float    *y,         /* y-coordinate of the particles             */
   int      Lx,         /* x-pixel size of the BMP file              */
   int      Ly,         /* y-pixel size of the BMP file              */
   char     *filename,  /* raster 8 BMP data filename                */
   char     *palette    /* 256x3 pallete filename                    */
)
// --------------------------------------------------------------------
{
     int     j,   jx,   jy;
     static  char   *a;
     float   *X,    *Y;

     a = (char *) malloc( sizeof(float)*Lx*Ly);
     for(j = 0; j < Lx*Ly; j++) a[j] = (char) 120;

     if(numGPUs > 0) {
       X = (float *) malloc(n*sizeof(float));
       Y = (float *) malloc(n*sizeof(float));
       cudaMemcpy( X, x, n*sizeof(float), cudaMemcpyDeviceToHost );
       cudaMemcpy( Y, y, n*sizeof(float), cudaMemcpyDeviceToHost );
     }
     else { X = x;   Y = y; }

     for(j = 0; j < n ; j++) {

         jx = MIN2(MAX2(0, (int)(X[j]*(float)Lx)), Lx);
         jy = MIN2(MAX2(0, (int)(Y[j]*(float)Ly)), Ly);

         a[Lx*jy + jx] = (char) 250;
     }

     DFR8bmp(a, Lx, Ly, filename, palette);
     fprintf(stderr,"filename=%s\n",filename);

     free(a);
     if(numGPUs > 0) { free(X);  free(Y); }

     return;
}



int  set_numGPUs(int numGPUs, int argc, char **argv)
{
     int   usrGPUs;

     if(argc > 1) {
        if(strncmp(argv[1],"-gpu",4) == 0) usrGPUs = numGPUs;
        else if(strncmp(argv[1],"-cpu",4) == 0) usrGPUs = 0;
        else                               usrGPUs = numGPUs;
     }
     else  usrGPUs = numGPUs;

     return usrGPUs;
}
