#include <cuda.h>
#include <cutil.h>


// typedef  double  FLOAT;
typedef  float  FLOAT;


#define   MAX2(x, y) ((x) > (y) ? (x) : (y))
#define   MIN2(x, y) ((x) < (y) ? (x) : (y))


void   initial
// ====================================================================
//
// purpos     :  initial positions of the particles
//
// date       :  Mar 1, 2009
// programmer :  Takayuki Aoki
// place      :  Tokyo Institute of Technology
//
(
   int      numGPUs,    /* available number of GPU device            */
   int      n,          /* number of the particles                   */
   float    *x,         /* x-coordinate of the particles             */
   float    *y          /* y-coordinate of the particles             */
);
// --------------------------------------------------------------------


void   swap
// ====================================================================
//
// purpos     :  pointer swap *f and *g
//
// date       :  Mar 01, 2009
// programmer :  Takayuki Aoki
// place      :  Tokyo Institute of Technology
//
(
   float   **f,         /* array pointer f                           */
   float   **g          /* array pointer g                           */
);
// --------------------------------------------------------------------


void   malloc_variables
// ====================================================================
//
// purpos     :  dynamic memory allocation (x, y, xn, yn)
//
// date       :  Mar 01, 2009
// programmer :  Takayuki Aoki
// place      :  Tokyo Institute of Technology
//
(
   int      numGPUs,    /* available number of GPU device            */
   int      n,          /* number of particles                       */
   float    **x,        /* x-coordinate of the particles             */
   float    **y,        /* y-coordinate of the particles             */
   float    **xn,       /* updated x-coordinate of the particles     */
   float    **yn        /* updated y-coordinate of the particles     */
);
// --------------------------------------------------------------------


void   DFR8bmp
// ====================================================================
//
// purpos     :  generation of 8-bit BMP format file (2-Dimensional)
//
// date       :  Jul 29, 2004
// programmer :  Takayuki Aoki
// place      :  Tokyo Institute of Technology
//
(
   char     *a,            /* data array to be output                */
   int      nx,            /* array size in the x-direction          */
   int      ny,            /* array size in the y-direction          */
   char     *filename,     /* raster 8 HDF data filename             */
   char     *palette       /* 256x3 pallete filename                 */
);
// --------------------------------------------------------------------


void   r8img_bmp
// ====================================================================
//
// purpos     :  generation of BMP image file (2-Dimensional)
//
// date       :  Mar 01, 2009
// programmer :  Takayuki Aoki
// place      :  Tokyo Institute of Technology
//
(
   int      numGPUs,    /* available number of GPU device            */
   int      n,          /* number of particles                       */
   float    *x,         /* x-coordinate of the particles             */
   float    *y,         /* y-coordinate of the particles             */
   int      Lx,         /* x-pixel size of the BMP file              */
   int      Ly,         /* y-pixel size of the BMP file              */
   char     *filename,  /* raster 8 BMP data filename                */
   char     *palette    /* 256x3 pallete filename                    */
);
// --------------------------------------------------------------------


void   ppush
// ====================================================================
//
// purpos     :  2-dimensional particle push by 4-stage Runge-Kutta
//               time integration
//
// date       :  May 12, 2009
// programmer :  Takayuki Aoki
// place      :  Tokyo Institute of Technology
//
(
   int      numGPUs,    /* available number of GPU device            */
   int      np,         /* number of the particles                   */
   float    *x,         /* x-coordinate of the particles             */
   float    *y,         /* y-coordinate of the particles             */
   float    *xn,        /* updated x-coordinate of the particles     */
   float    *yn,        /* updated y-coordinate of the particles     */
   float    t,          /* time                                      */
   float    dt          /* time step interval                        */
);
// --------------------------------------------------------------------


int     set_numGPUs(int numGPUs, int argc, char **argv);
FLOAT   cpu_time();
